#!/usr/bin/env python

from socrates_workshop.srv import *
import rospy

Inf = +999999

MINIMAX_NODE_NAME = 'minimax_server'
MINIMAX_TOPIC_NAME = 'minimax'





def h1(s, player, indeces):
	return all([s[i] == player for i in indeces])

def h2(s,p):
	return h1(s,p,[0,1,2]) or h1(s,p,[3,4,5]) or h1(s,p,[6,7,8]) or h1(s,p,[0,3,6]) or h1(s,p,[2,4,7]) or h1(s,p,[3,5,8]) or h1(s,p,[0,4,8]) or h1(s,p,[2,4,6])

def getHeuristicValue(s, player):
	return 10*h2(s, player) + sum(1 for x in range(len(s)) if s[x] == 0)

def getOpponent(player):
	return -1 * player

def isTerminal(s, player):
	opp = getOpponent(player)
	return all([cell != 0 for cell in s]) or h2(s, player) or h2(s, opp)

def possibleActions(s, player):
	print [x for x in range(len(s)) if s[x] == 0]
	return [x for x in range(len(s)) if s[x] == 0]


def executeAction(s, action, player):
	s[action] = player
	return s


def minimax(state, player, me, depth):
	selectedAction = None
	opp = getOpponent(player)

	if isTerminal(state, player) or depth == 0:
		return [getHeuristicValue(state, player), selectedAction]

	if player == me:
		alpha = -Inf
		print "-1-"
		for action in possibleActions(state, player):
			s1 = executeAction([x for x in state], action, player)
			print s1
			childScore = minimax(s1, opp, me, depth - 1)[0]
			if childScore > alpha:
				alpha = childScore
				selectedAction = action
	else:
		print "-2-"
		alpha = Inf
		for action in possibleActions(state, player):
			s1 = executeAction(state, action, player)
			print s1
			childScore = minimax(s1, opp, me, depth - 1)[0]
			if childScore < alpha:
				alpha = childScore
				selectedAction = action

	return [alpha, selectedAction]

def handleComputeMiniMax(req):
	cells = [x for x in req.board]
	depth = req.max_depth
	me = 1
	alpha, selectedAction = minimax(cells, me, me, depth)
	print alpha
	return MiniMaxResponse(selectedAction)

def minimaxServer():
	rospy.init_node(MINIMAX_NODE_NAME)
	s = rospy.Service(MINIMAX_TOPIC_NAME, MiniMax, handleComputeMiniMax)
	rospy.spin()

if __name__ == "__main__":
	minimaxServer()
