#!/usr/bin/env python
import rospy
import tic_tac_toe_common as ttt
from std_msgs.msg import *

from socrates_workshop.srv import MiniMax, Board, IntInt

NODE_NAME = "node_main"


TOPIC_ASK_HELP		 	= "topic_ask_help"
TOPIC_BOARD_STATE	 	= "topic_board_state"
TOPIC_MINIMAX		 	= "topic_minimax_service"
TOPIC_DIFFICULTY 		= "topic_difficulty"

TOPIC_ACTION_CALLBACK	 = "topic_gripper_callback"


this = sys.modules[__name__]

this.board_service_proxy 	= None
this.minimax_service_proxy 	= None
this.gripper_subscriber		= None
this.difficulty_service_proxy = None



def helpUser(nextAction, board):
	rospy.loginfo("I suggest action %s for board %s", nextAction, board)

def helpCallback(msg):
	id_human = ttt.getUserId()
	id_ai = ttt.getAiId()

	msg_board = this.board_service_proxy(id_human, id_ai)

	depth = 10
	msg_action = this.minimax_service_proxy(msg_board.board, depth, id_human, id_ai)

	helpUser(msg_action.action, msg_board.board)


"""
Callback function for the gripper movement
Depending of current difficulty we execute different routines
"""
def callbackGripperActionExecuted(msg):
	success = msg.data

	# We query the current difficulty level
	msg_difficulty = this.difficulty_service_proxy(0)
	difficulty = msg_difficulty.data

	if difficulty == 1:
		helpCallback(None)

	if difficulty == 4:
		# countdown
		pass
	
    
def init_this_node():

	rospy.init_node(NODE_NAME, anonymous=True)

	topic = ttt.concatTopicName(TOPIC_ASK_HELP)
	s = rospy.Subscriber(topic, Empty, helpCallback)

	topic = ttt.concatTopicName(TOPIC_BOARD_STATE)
	this.board_service_proxy = rospy.ServiceProxy(topic, Board)

	topic = ttt.concatTopicName(TOPIC_MINIMAX)
	this.minimax_service_proxy = rospy.ServiceProxy(topic, MiniMax)



	# Difficulty service to query the current level of difficulty
	topic = ttt.concatTopicName(TOPIC_DIFFICULTY)
	this.difficulty_service_proxy = rospy.ServiceProxy(topic, IntInt)

	# Subscriber on the gripper callback invoked each time the gipper concludes an action
	topic = ttt.concatTopicName(TOPIC_ACTION_CALLBACK)
	this.gripper_subscriber = rospy.Subscriber(topic, Bool, callbackGripperActionExecuted)


	rospy.spin()

if __name__ == '__main__':
	init_this_node()
