#!/usr/bin/env python
import rospy
import tic_tac_toe_common as ttt
from std_msgs.msg import *
from std_srvs.srv import *
from socrates_workshop.srv import *

NODE_NAME = "node_main"


TOPIC_BOARD_STATE 		= "topic_board_state"
TOPIC_PERFORM_ACTION 	= "topic_perform_robot_action"
TOPIC_MINIMAX 			= "topic_minimax_service"

TOPIC_ACTION_CALLBACK 	= "topic_gripper_callback"
TOPIC_GRIPPER_ACTION 	= "topic_gripper_action"


this = sys.modules[__name__]

# Services
this.action_service			= None

# Service proxies
this.board_service_proxy 	= None
this.minimax_service_proxy 	= None

this.gripper_action_publisher = None

# Other variables
this.gripper_moving 		= False


"""
Helper function to call the minimax service
"""
def callMinimax():
	id_h = ttt.getUserId()
	id_ai = ttt.getAiId()

	resp = this.board_service_proxy(id_h,id_ai)
	current_state = resp.board

	depth = 10
	resp = this.minimax_service_proxy(current_state, depth, id_ai, id_h)

	return resp.action


"""
Callback function for the gripper movement
"""
def callbackGripperActionExecuted(msg):
	success = msg.data

	# The gripper stopped moving
	this.gripper_moving = False
	

"""
Invoked each time service on TOPIC_PERFORM_ACTION is called

"""
def actionCallback(msg):

	# If gripper is still performing the previous action 
	# we do nothing
	if this.gripper_moving:
		pass

	else:
		# We set the gripper as moving
		this.gripper_moving = True

		# We find the correct action to perform
		action = callMinimax()

		# The action id (between 1 and 9) is published
		# for the gripper node
		this.gripper_action_publisher.publish(action)

	return EmptyResponse()

    
def init_this_node():

	rospy.init_node(NODE_NAME, anonymous=True)
	
	# Service that provides the action execution
	topic = ttt.concatTopicName(TOPIC_PERFORM_ACTION)
	this.action_service = rospy.Service(topic, Empty, actionCallback)

	# Board service to query the state of the board
	topic = ttt.concatTopicName(TOPIC_BOARD_STATE)
	this.board_service_proxy = rospy.ServiceProxy(topic, Board)

	# Minimax service
	topic = ttt.concatTopicName(TOPIC_MINIMAX)
	this.minimax_service_proxy = rospy.ServiceProxy(topic, MiniMax)

	# We will publish on the gripper topic the action to execute (actions have ids 1 to 9)
	topic = ttt.concatTopicName(TOPIC_GRIPPER_ACTION)
	this.gripper_action_publisher = rospy.Publisher(topic, Int16, queue_size=10)


	# Subscriber on the gripper callback invoked each time the gipper concludes an action
	topic = ttt.concatTopicName(TOPIC_ACTION_CALLBACK)
	this.gripper_subscriber = rospy.Subscriber(topic, Bool, callbackGripperActionExecuted)

	rospy.spin()

if __name__ == '__main__':
	init_this_node()
