#!/usr/bin/env python
import rospy
import tic_tac_toe_common as ttt

from socrates_workshop.srv import *

Inf = +999999

NODE_NAME = 'minimax_server'


TOPIC_MINIMAX = "topic_minimax_service"



def h1(s, player, indeces):
	return all([s[i] == player for i in indeces])

def h2(s,p):
	return h1(s,p,[0,1,2]) or h1(s,p,[3,4,5]) or h1(s,p,[6,7,8]) or h1(s,p,[0,3,6]) or h1(s,p,[1,4,7]) or h1(s,p,[2,5,8]) or h1(s,p,[0,4,8]) or h1(s,p,[2,4,6])


def getOpponent(player, p1, p2):
	if player == p1:
		return p2
	if player == p2:
		return p1
	return 0

def getHeuristicValue(s, player, p1, p2):
	opp = getOpponent(player, p1, p2)
	return 10*h2(s, player) + sum(1 for x in range(len(s)) if s[x] == 0) - 10 * h2(s, opp)


def isTerminal(s, player, p1, p2):
	opp = getOpponent(player, p1, p2)
	return all([cell != 0 for cell in s]) or h2(s, player) or h2(s, opp)

def possibleActions(s, player):
	return [x+1 for x in range(len(s)) if s[x] == 0]


def executeAction(s, action, player):
	s[action-1] = player
	return s

def minimax(state, player, p1, p2, depth):
	selectedAction = None
	opp = getOpponent(player, p1, p2)

	if isTerminal(state, player, p1, p2) or depth == 0:
		h = getHeuristicValue(state, p1, p1, p2)
		return [h, selectedAction]

	if player == p1:
		alpha = -Inf
		for action in possibleActions(state, player):
			s1 = executeAction([x for x in state], action, player)
			childScore = minimax(s1, opp, p1, p2, depth - 1)[0]
			if childScore > alpha:
				alpha = childScore
				selectedAction = action
	else:
		alpha = Inf
		for action in possibleActions(state, player):
			s1 = executeAction([x for x in state], action, player)
			childScore = minimax(s1, opp, p1, p2, depth - 1)[0]
			if childScore < alpha:
				alpha = childScore
				selectedAction = action

	return [alpha, selectedAction]

def handleComputeMiniMax(req):
	cells = [x for x in req.board]
	depth = req.max_depth
	p1 = req.id_player
	p2 = req.id_opponent

	alpha, selectedAction = minimax(cells, p1, p1, p2, depth)

	return MiniMaxResponse(selectedAction)

def minimaxServer():
	rospy.init_node(NODE_NAME)

	s = rospy.Service(ttt.concatTopicName(TOPIC_MINIMAX), MiniMax, handleComputeMiniMax)
	rospy.spin()

if __name__ == "__main__":
	minimaxServer()
