#!/usr/bin/env python

# import image processing definitions
from impprocess_def import *
from obj_recogkinect import *
import cv2
from math import pi, sin, cos, atan2, sqrt
import numpy as np


class ImgProcssXO():
    ''' Image processing class to play tic-tac-toe '''

    def __init__(self, size, low_canny=lowerCanny, up_canny=upperCanny, rho=Rho, theta=Theta,
                 hough_Thresh=houghThreshold, rho_Div=RhoDivisor, theta_Div=ThetaDivisor):

        ##set height and width of image
        self.width = size[1]
        self.height = size[0]
        ##lower and upper threshold of canny for edge detection
        self.lcanny = low_canny
        self.ucanny = up_canny
        # Hough's distance and angular resolution, accumulator value
        self.disHog = 1.0 * rho / rho_Div
        self.anghog = 1.0 * theta / theta_Div
        self.threshHog = hough_Thresh

#    def draw_lines(self, image, lines):
#
#        if len(lines[0]) == 2:
#            for rho, theta in lines:
#                a = np.cos(theta)
#                b = np.sin(theta)
#               x0 = a * rho
#               y0 = b * rho
#               x1 = int(x0 + 1000 * (-b))
#               y1 = int(x0 + 1000 * (a))
#               x2 = int(x0 - 1000 * (-b))
#               y2 = int(x0 - 1000 * (a))
#               cv2.line(image, (x1,y1), (x2,y2), (0,0,255),2 )
#           return image
#       elif len(lines[0])==4:
#       for _, _, rho, theta in lines:
#               a = np.cos(theta)
#               b = np.sin(theta)
#               x0 = a * rho
#               y0 = b * rho
#               x1 = int(x0 + 1000 * (-b))
#                y1 = int(x0 + 1000 * (a))
#                x2 = int(x0 - 1000 * (-b))
#                y2 = int(x0 - 1000 * (a))
#                cv2.line(image, (x1, y1), (x2, y2), (0,0,255), 2)
#            return image

#        return image

    def preprossline(self,image):
        '''Using OpenCv to first convert the image to grayscale, applying canny
        edge detector for edge extraction and then apply hough transformation
        to extract lines.'''
        self.img_gry = cv2.IMREAD_GRAYSCALE(image)
        cv2.imshow("Grayscale",self.img_gry)
        #Extract edges with canny
        edges = cv2.Canny(self.img_gry,self.lcanny,self.ucanny)
        cv2.imshow("Edges", edges)
        cv2.waitKey(1)
        #Apply Houghtrasnform
        try:
            lines = cv2.HoughLines(edges, self.disHog,self.anghog)[0]
            lines_img = self.draw_lines(image.copy(), lines)
            cv2.imshow("Lines", lines_img)
            cv2.waitKey(1)
        except:
            print("No lines found")
            return []
        return lines

    def getlineEndpnts(self, lines):
        '''Obtain endpoints of all the lines and then store them in pts1 and pts2'''
        pts1 = []
        pts2 = []
        flg_line = [0]*len(lines)
        #for loop for all the lines
        for i in range(0, len(lines)):
            #params of lines
            rho = lines[i][1]
            theta = lines[i][1]
            #if line is horizontal
            if theta == 0:
                pts1.append((rho,0))
                pts2.append((rho, self.width))
            #if line is vertical
            elif theta == pi/2:
                pts1.append((0, rho))
                pts2.append((self.height, rho))
            else:
                print ('Line is not in the image')

            flg_line[i]=1

        return pts1, pts2

    def getBoundLine(self, P, corner_pts):
        '''Obtain the lines bounding the playing field'''
        #calculate coordinates of corners in image
        self.corners = np.dot(P, corner_pts)
        self.corners = self.corners / self.corners[2]
        #bound lines through corners
        self.boundingLines = []
        for i in range(4):
            pt2 = (cv2.boundingRect(self.corners[0][i-1]%4), cv2.boundingRect(self.corners[1][i-1]%4))
            pt1 = (cv2.boundingRect(self.corners[0][i]), cv2.boundingRect(self.corners[1][i]))
            self.boundingLines.append((pt2,pt1))
        return

    def getPntLnth(self, pt1, pt2):
        '''Returns distance between two 2D points PYTHAGORAS THEOREM'''
        val = sqrt( ((pt1[0] - pt2[0] ** 2) + (pt1[1] - pt2[1] ** 2)) )
        return val

    def getMeanPnt(self, points):
        '''Returns mean value of x and y component of points'''

        x = []
        y = []
        for point in points:
            x.append(point[0])
            y.append(point[1])
        return (round(sum(x)/len(x)), round(sum(y)/len(y)))

    def mrgeEndPnts(self, lines, reltol):
        pts1,pts2 = self.getlineEndpnts(lines)
        lines_mrgd = []
        flg_line = [0] * len(pts1)
        #loop through everyline
        for i in range(min(len(pts1), len(pts2))):
            pts1_near = []
            pts2_near = []
            if flg_line[i] == 0:
                pts1_near.append(pts1[i])
                pts2_near.append(pts2[i])

                for j in range(i, min(len(pts1), len(pts2))):
                    ## if line j was not already merged with other line
                    if flg_line[j] == 0 and float(self.getPntLnth(pts1[i], pts1[j]) / self.width) < reltol and float(
                                    self.getPntLnth(pts2[i], pts2[j]) / self.width) < reltol:
                    ## if both ends of line j are near to endings of line i, add line j to be merged with line i and flag line j as processed
                        pts1_near.append(pts1[j])
                        pts2_near.append(pts2[j])
                        flg_line[j] = 1
                    ##check for other combination of lines
                    elif float(self.getPntLnth(pts1[i], pts2[j]) / self.width) < reltol and float(
                                    self.getPntLnth(pts2[i], pts1[j]) / self.width) < reltol:
                        pts1_near.append(pts2[j])
                        pts2_near.append(pts1[j])
                        flg_line[j] = 1

            ## find mean of points of all the lines which are close to each other
            pt1 = self.getMeanPnt(pts1_near)
            pt2 = self.getMeanPnt(pts2_near)
            #calculate rho and theta for new merged lines
            num = float(pt1[0] - pt2[0])
            den = float(pt1[1] - pt2[0])
            theta_m = atan2(num, den)
            if theta_m < 0:
                ## shift to [0,2pi]
                theta_m = theta_m + 2 * pi
            if theta_m == 0:  # horizontal line, pt1[0] and pt2[0] should be the same and equal to rho
                rho_m = float(pt1[0] + pt2[0]) / 2
            elif theta_m == pi / 2:  # vertical line, p1[1] and pt2[1] sould be the same and equal to rho
                rho_m = float(pt1[1] + pt2[1]) / 2
            else:
                ## line is neither horizontal nor vertical, calculate rho
                rho_m = sin(theta_m) * (pt1[1] * pt2[0] - pt1[0] * pt2[1]) / (pt2[0] - pt1[0] + 1e-9)
                ## add new line to the list of merged lines
            lines_mrgd.append((rho_m, theta_m))
            ## flag line i as processed
            flg_line[i] = 1
            ## return merged lines
        return lines_mrgd

    def getContours(self, cv2_img):
        '''Extract contours of noughts and crosses
        by applying it on image generated from opencv'''
        #find all contours inside tic-tac-toe board
        img_gry = cv2.IMREAD_GRAYSCALE(cv2_img)
        (cnts, _) = cv2.findContours(img_gry.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        for (i, c) in enumerate(cnts):
            # Compute area of contours
            area = cv2.contourArea(c)
            (x, y, w, h) = cv2.boundingRect(c)
            hull = cv2.convexHull(c)
            hullArea = cv2.contourArea(hull)
            solidity = area / float(hullArea)
            cnt_o = cnts[0]
            cnt_x = cnts[1]
            if solidity > 0.9 :
                contour_o = cv2.drawContours(cv2_img, [cnt_o], 0, (0, 255, 0), 3)
            elif solidity > 0.5:
                contour_x = cv2.drawContours(cv2_img, (cnt_x,), 0, (0, 255, 0), 3)
        return contour_x,contour_o, (cv2.imshow('Output',cv2_img))


    def isInsideField(self,centroid, boundg_line):
        '''Return true if centroid is inside the required area.'''
        d1 = centroid[0]*cos(boundg_line[0][3]+centroid[1]*sin(boundg_line[0][3])-boundg_line[0][2])
        d2 = centroid[0]*cos(boundg_line[1][3]+centroid[1]*sin(boundg_line[1][3])-boundg_line[1][2])
        d3 = centroid[0]*cos(boundg_line[2][3]+centroid[1]*sin(boundg_line[2][3])-boundg_line[2][2])
        d4 = centroid[0]*cos(boundg_line[3][3]+centroid[1]*sin(boundg_line[3][3])-boundg_line[3][2])
        if d1 < 0 and d2 < 0 and d3 < 0 and d4 < 0 :
            return True
        return False

    def getCntroids(self, cv2_img):
        '''Gives centroid of objects which are inside the playing field'''
        contour_o = self.getContours_O(cv2_img)
        o = []
        if contour_o:
            for contour in contour_o:
                if cv2.contourArea(contour) > 250:
                    mmnts = cv2.moments(contour)
                    centroid = (int(mmnts['m10']/mmnts['m00']), int(mmnts['m01']/mmnts['m00']))
                    if self.isInsideField(centroid,self.boundg_line):
                        o.append(centroid)

        contour_x = self.getContours_X(cv2_img)
        x = []
        if contour_x:
            for contour in contour_x:
                if cv2.contourArea(contour) > 250:
                    mmnts = cv2.moments(contour)
                    centroid = (int(mmnts['m10'] / mmnts['m00']), int(mmnts['m01'] / mmnts['m00']))
                    if self.isInsideField(centroid, self.boundg_line):
                        x.append(centroid)

        return o,x

    def getGameState(self, cv2_img, newcv2_img, indexedLines, cross, dougnuts):

        '''Here with surf detector we determine the game state'''

        state = ['-']*9
        board = [[-1,-1,-1],[-1,-1,-1],[-1,-1,-1]]
        img_prev = []
        img_new = []
        surf = cv2.xfeatures2d.SURF_create(3000)

        if img_prev == cv2_img and img_new == newcv2_img:
            (kp1, des1) = surf.detectAndCompute(img_prev,None)
            (kp, des) = surf.detectAndCompute(img_new, None)
            img1 = cv2.drawKeypoints(img_prev, kp, None, (255,0,0), 4)
            img2 = cv2.drawKeypoints(img_new, kp, None, (255,0,0), 4)
            for elements in img2:
                if dougnuts:
                    (x_cord, y_cord) = self.getContours_O(cv2_img)
                    state[x_cord*3+y_cord] = 'o'
                    board[x_cord][y_cord] = 1
                elif cross:
                    (x_cord, y_cord) = self.getContours_O(cv2_img)
                    state[x_cord*3+y_cord] = 'x'
                    board[x_cord][y_cord] = 1
                else:
                    print "No element found"
        return state,board




    



