#!/usr/bin/env python

import rospy
import os
import tic_tac_toe_common as ttt

from socrates_workshop.srv import *
from std_msgs.msg import Bool, Int16
from std_srvs.srv import Empty



TOPIC_BOARD_STATE = "topic_board_state"
TOPIC_PERFORM_ACTION = "topic_perform_robot_action"

TOPIC_GRIPPER_ACTION = "topic_gripper_action"
TOPIC_ACTION_CALLBACK = "topic_gripper_callback"

clearTerminal = lambda: os.system('clear')

this = sys.modules[__name__]

this.callback_publisher = None
this.robot_action_proxy = None

this.refreshing = False
this.board = [0,0,0,0,0,0,0,0,0]
this.last_move = 0

def h1(s, player, indeces):
	return all([s[i] == player for i in indeces])

def h2(s,p):
	return h1(s,p,[0,1,2]) or h1(s,p,[3,4,5]) or h1(s,p,[6,7,8]) or h1(s,p,[0,3,6]) or h1(s,p,[1,4,7]) or h1(s,p,[2,5,8]) or h1(s,p,[0,4,8]) or h1(s,p,[2,4,6])

def gameFinished():

	id_human = ttt.getUserId()
	id_ai = ttt.getAiId()
	indeces = [0,1,2,3,4,5,6,7,8]

	s = this.board
	
	if h2(s, id_human) or h2(s, id_ai) or all( [s[i] != 0 for i in indeces] ):
		return True
	else:
		return False
	

def newGame():
	this.board = [0,0,0,0,0,0,0,0,0]
	this.last_move = 0
	return this.board

def getPiece(p):
	if p == 0:
		return ' '
	elif p == 1:
		return 'X'
	else:
		return 'O'


def refreshView():
	if not this.refreshing:
		this.refreshing = True
		s = this.board
		clearTerminal()
		print "let's play tic tac toe!!"
		print " %s | %s | %s" % (getPiece(s[0]), getPiece(s[1]), getPiece(s[2])) 
		print "-----------"
		print " %s | %s | %s" % (getPiece(s[3]), getPiece(s[4]), getPiece(s[5])) 
		print "-----------"
		print " %s | %s | %s" % (getPiece(s[6]), getPiece(s[7]), getPiece(s[8])) 
		print "Enter your desired piece position in range [1..9]"
		this.refreshing = False


def setPiece(player, position):
	if this.board[position-1] != 0:
		return False
	this.board[position-1] = player
	this.last_move = player
	if gameFinished():
		newGame()

	refreshView()
	return True

def callbackBoardRequest(req):
	return BoardResponse(this.board)

def callbackOnGripperAction(msg):
	action = msg.data
	setPiece(ttt.getAiId(), action)
	this.callback_publisher.publish(True)
	

def tic_tac_toe_node():
	rospy.init_node("tic_tac_toe")


	id_human = ttt.getUserId()

	exit = False

	topic = ttt.concatTopicName(TOPIC_BOARD_STATE)
	s = rospy.Service(topic, Board, callbackBoardRequest)

	topic = ttt.concatTopicName(TOPIC_GRIPPER_ACTION)
	s = rospy.Subscriber(topic, Int16, callbackOnGripperAction)

	topic = ttt.concatTopicName(TOPIC_ACTION_CALLBACK)
	this.callback_publisher = rospy.Publisher(topic, Bool, queue_size=10)

	topic = ttt.concatTopicName(TOPIC_PERFORM_ACTION)
	this.robot_action_proxy = rospy.ServiceProxy(topic, Empty)


	clearTerminal()
	newGame()
	while not rospy.is_shutdown() and not exit:
		refreshView()
		while this.last_move == id_human:
			pass
		user_input = input()

		if user_input not in range(1, 10):
			exit = True

		if setPiece(id_human, user_input):
			this.robot_action_proxy()

	print exit
		

if __name__ == "__main__":
	tic_tac_toe_node()
